#!/usr/bin/env ruby

$strPath=File.split(`#{"which rqr.rb"}`.chomp)[0]+"/"

require $strPath+"qrcode.rb"
#require "./qrcode.rb"

$q=Qrcode.new

require 'qte'
require 'qpe'

include Qte
include Qpe

$codec = QTextCodec.codecForName("Shift-JIS")

$chrZero = "0"[0]

$iBlock=8

class RqrConf
  def initialize(strFile = "/home/zaurus/Settings/rqr.conf")
    @strConfFile = strFile
    @hDefaultConf = {
      "sizeFont" => "6",
      "sizeQr" => "0",
      "pathCurPix" => "/home/zaurus/Documents/rqr.jpeg"
    }
    @hConf=@hDefaultConf
    load
  end
  def load
    if FileTest.file?(@strConfFile)==false then
      save
      return @hConf
    end
    fp = File.new(@strConfFile,"r")
    fp.each do |line|
      line = line.chomp
      unless line =~ /^$/
        a = line.split("=",2)
        @hConf[a[0]]=a[1]
      end
    end
    @hDefaultConf.each do |k, v|
      unless @hConf.has_key?(k)
        @hConf[k]=v
      end
    end
    save
    return @hConf
  end
  def getConf
    @hConf
  end
  def setConf(h)
    @hConf=h
    save
  end
  def save
    fp = File.new(@strConfFile,"w")
    @hConf.each do |k, v|
      fp.print(k + "=" + v + "\n")
    end
    fp.close
  end
end

class RqrSetPixSizeDialog < QDialog
  def initialize
    super(nil, "RqrSetPixSizeDialog", true) # Modal Dialog
    setCaption(tr("サイズ設定"))
    vb=QVBox.new(self)
    vb.setFixedSize(200,120)
      @ckAuto=QCheckBox.new(tr("自動調節"),vb)
      QLabel.new(tr("　チェックしない場合、"),vb)
      QLabel.new(tr("↓サイズを指定します。"),vb)
      hb=QHBox.new(vb)
      @sbSize=QSpinBox.new(hb)
      QLabel.new(tr("ピクセル"),hb)
      @sbSize.setMinValue(0)
      @sbSize.setMaxValue(400)
      connect(@ckAuto,QSIGNAL("stateChanged(int)"),self,"tglCheck(i)")
  end
  def exec
    h=$conf.getConf
    if h["sizeQr"]=="0" then
      @ckAuto.setChecked(true)
      @sbSize.setValue(100)
      @sbSize.setEnabled(false)
    else
      @ckAuto.setChecked(false)
      @sbSize.setValue(h["sizeQr"].to_i)
      @sbSize.setEnabled(true)
    end
    super
  end
  def getConf
    h=$conf.getConf
    if @ckAuto.isChecked then
      h["sizeQr"]="0"
    else
      h["sizeQr"]=@sbSize.value.to_s
    end
    h
  end
  def tglCheck(i)
    case i
    when 0
      @sbSize.setEnabled(true)
    when 2
      @sbSize.setEnabled(false)
    end
  end
end

class RqrDialog < QDialog
  def initialize
    super(nil, "RqrDialog", true) # Modal Dialog
    setFixedSize(640,410)
    setCaption(tr("RQR"))
    @qrCanvas=RqrCanvas.new(self)
    @qrCanvas.setFixedSize(size)
  end
  def getCanvas
    @qrCanvas
  end
end

class RqrCanvas < QWidget
  def initialize(parent)
    super(parent)
    setPalette(QPalette.new(QColor.new(255,255,255)))
    @strQr=""
    @qpBuf=QPixmap.new(size)
    @qpBuf.fill(QColor.new(255,255,255))
    @p=QPainter.new
    catchEvent
  end
  def drawQr(strQr)
    @strQr=strQr
    if @strQr!="" then
      astrQr = @strQr.split(/\n/)
      widthQr = astrQr[0].length

      h=$conf.getConf

      if h["sizeQr"]=="0" then

        s=widthQr*$iBlock
        i=$iBlock
        j=$iBlock
        if s>height then
          i=height.to_f/widthQr.to_f
          j=i.ceil
          s=widthQr*i
        end
        if s>width then
          i=width.to_f/widthQr.to_f
          j=i.ceil
          s=widthQr*i
        end

      else

        s=h["sizeQr"].to_i
        i=s.to_f/widthQr.to_f
        j=i.ceil

      end

      @qpBuf.resize(s,s)

      @qpBuf.fill(QColor.new(255,255,255))
      @p.begin(@qpBuf)
      @p.setBrush(QColor.new(0,0,0))
      @p.setPen(SolidLine)

      y = 0
      while y < widthQr
        strOne = astrQr[y]
        x = 0
        while x < widthQr
          if strOne[x]!=$chrZero then
            @p.drawRect(x*i, y*i, j, j)
          end
          x += 1
        end
        y += 1
      end
      @p.end
      h=$conf.getConf
      @qpBuf.save(h["pathCurPix"],"JPEG")
    end
  end
  def paintEvent(e)
    bitBlt(self,width/2 - @qpBuf.width/2, height/2 - @qpBuf.height/2 ,@qpBuf)
  end
end

class RqrEdit < QMultiLineEdit
  def initialize(p)
    super(p)
    setWordWrap(WidgetWidth)
    setWrapPolicy(Anywhere)

    @aFontSize=[10,12,16,18,20,24,36,48,72]
    setupFontSize
    catchEvent
  end

  def setupFontSize
    hConf=$conf.getConf
    setFont(QFont.new("lcfont",@aFontSize[hConf["sizeFont"].to_i]))
  end

  def updownFont(i)
    hConf=$conf.getConf
    j=hConf["sizeFont"].to_i + i
    if j < 0 or j >= @aFontSize.size then
      return
    end
    hConf["sizeFont"]=j.to_s
    $conf.setConf(hConf)
    setupFontSize
  end

  def keyPressEvent(e)
    case e.key
    when Qt::Key_Tab
      $frmMain.execQr
      e.ignore
    when 4154
      $frmMain.setFocus2Menu
      e.ignore
    end
  end
end

class FormMain < QMainWindow
  def initialize
    super
    setCaption("RQR")
    vb=QVBox.new(self)
      @leStr = RqrEdit.new(vb)
      @btnExec = QPushButton.new(tr("QRコードに変換"),vb)
    setCentralWidget(vb)
    @strLastData=""
    @strLastQr=""
    @strLastSize=""
    @hMenuId=Hash.new
    @mbar=QMenuBar.new(self)
    @hPop=Hash.new
    ["ファイル","フォント"].each do |e|
      @hPop[e]=QPopupMenu.new
      @mbar.insertItem(tr(e),@hPop[e])
    end
    p=@hPop["ファイル"]
    @hMenuId["setPixPath"]=p.insertItem(tr("画像ファイル保存先変更"))
    @hMenuId["setPixSize"]=p.insertItem(tr("画像サイズ変更"))
    @hMenuId["execQr"]=p.insertItem(tr("QRコードに変換"))
    p=@hPop["フォント"]
    @hMenuId["upFontSize"]=p.insertItem(tr("もっと大きく"))
    @hMenuId["downFontSize"]=p.insertItem(tr("もっと小さく"))
    
    connect(@btnExec,QSIGNAL("clicked()"),self,"execQr")
    connect(@hPop["ファイル"],QSIGNAL("activated(int)"),self,"activateMenu(id)")
    connect(@hPop["フォント"],QSIGNAL("activated(int)"),self,"activateMenu(id)")
    catchEvent
  end

  def activateMenu(id)
    case id
    when @hMenuId["execQr"]
      execQr
    when @hMenuId["upFontSize"]
      @leStr.updownFont(1)
    when @hMenuId["downFontSize"]
      @leStr.updownFont(-1)
    when @hMenuId["setPixSize"]
      if $sizeDialog.exec !=0 then
        $conf.setConf($sizeDialog.getConf)
      end
    when @hMenuId["setPixPath"]
      selectFileName
    end
  end

  def selectFileName
    h = $conf.getConf
    if $fileDialog.showDialog(h["pathCurPix"]) !=0 then
      h["pathCurPix"]=$fileDialog.getSelectedFile
      $conf.setConf(h)
    end
  end

  def setFocus2Menu
    @mbar.setFocus
  end

  def execQr
    strData = $codec.fromUnicode(@leStr.text).data.gsub(/\n/,"\r\n")
    h=$conf.getConf
    if strData==@strLastData and h["sizeQr"]==@strLastSize then
        $rqrDialog.exec
    elsif strData==@strLastData and h["sizeQr"]!=@strLastSize then
        $rqrCanvas.drawQr(@strLastQr)
        $rqrDialog.exec
    else
      setCaption(tr("RQR 変換中..."))
      @pCont = Proc.new {
        @strLastData=strData
        @strLastSize=h["sizeQr"]
        $q.set_qrcode_version(0)
        $q.set_qrcode_error_correct("M")
        @strLastQr=$q.make_qrcode(strData)
        $rqrCanvas.drawQr(@strLastQr)
        setCaption(tr("RQR"))
        $rqrDialog.exec
      }
      startTimer(0)
    end
  end

  def timerEvent(e)
    killTimers
    @pCont.call
  end
end

class RqrFileDialog < QDialog
  def initialize
    super(nil, "RqrFileDialog", true) # Modal Dialog
    setCaption(tr("保存ファイル名指定"))
    vb=QVBox.new(self)
    vb.setFixedSize(570,300)

    @dir = "/home/samba/Main_Memory"
    
    @dir.sub!(/\/usr\/mnt.rom\/cf/,"/home/samba/CF_Card")
    @dir.sub!(/\/usr\/mnt.rom\/card/,"/home/samba/SD_Card")
    @dir.sub!(/\/home\/zaurus\/Documents/,"/home/samba/Main_Memory")
    @dir.chomp!()
    @fname = ""

    @list = QListView.new(vb)
    @list.addColumn("File Name",250)
    @list.addColumn("Size",100)
    @list.addColumn("DateTime",200)

    @list.setColumnAlignment(0,1)
    @list.setColumnAlignment(1,2)
    @list.setColumnAlignment(2,2)

    @f = Array.new()
    
    @le = QLineEdit.new(vb)
    
    connect( @list, QSIGNAL("returnPressed(QListViewItem*)"), self, 'selectFile')
  end

  def showDialog(df)
    @dir = File.dirname(df)
    @fname = File.basename(df)
    Dir.chdir(@dir)
    @le.setText(tr(@fname))
    showFiles
    exec
  end

  def showFiles
    @list.clear
    getDir
    i = 0
    setCaption(@dir)
    while i < @f.size
      fo = File.stat(@f[i])
      fs = fo.size / 1024
      fss = if fo.directory? then
              ""
            else
              fs.to_s + "Kb "
            end
      ft = fo.mtime.to_s
      li = QListViewItem.new(@list,@f[i],fss,ft)
      @list.setSelected(li, true) if @f[i] == @fname
      i += 1
    end
#    @fname = ""
  end

  def selectFile(selectedItem)
    sf = selectedItem.text(0).to_str
    if FileTest.directory?(sf)
      Dir.chdir(sf)
      @dir =Dir.pwd
      @dir = "/home/samba" if @dir == "/home/zaurus"
      @dir = "/home/samba" if @dir == "/mnt"
      @dir = "/home/samba" if @dir == "/usr/mnt.rom"
      Dir.chdir(@dir)
#      @fname = "."
      showFiles
    else
      @fname = sf
      @le.setText(tr(@fname))
    end
  end

  def getDir
    @f.clear
    @f = Dir.entries("./")
    @f.sort!
  end
  
  def getSelectedFile
    @dir + "/" + @fname
  end
end

a = QPEApplication.new([$0] + ARGV)
QPEApplication.setFont(QFont.new("lcfont",18))
a.setDefaultCodec(QTextCodec.codecForName("UTF-8"))
$conf = RqrConf.new
$sizeDialog = RqrSetPixSizeDialog.new
$fileDialog = RqrFileDialog.new
$frmMain = FormMain.new
$rqrDialog = RqrDialog.new
$rqrCanvas = $rqrDialog.getCanvas
a.showMainWidget($frmMain)
a.exec

